@@ -11,7 +11,7 @@ use Raisin::Response;
 use Raisin::Routes;
 use Raisin::Util;
 
-our $VERSION = '0.67';
+our $VERSION = '0.68';
 
 sub new {
     my ($class, %args) = @_;
@@ -168,7 +168,7 @@ sub psgi {
 
         # Validation and coercion of declared params
         if (!$req->prepare_params($route->params, $route->named)) {
-            $res->render_error(400, 'Invalid params');
+            $res->render_error(400, 'Invalid parameters');
             return $res->finalize;
         }
 
@@ -301,25 +301,30 @@ Raisin - a REST API micro framework for Perl.
 
     use utf8;
 
+    use FindBin;
+    use lib "$FindBin::Bin/../../lib";
+
     use List::Util qw(max);
     use Raisin::API;
-    use Types::Standard qw(Any Int Str);
+    use Types::Standard qw(HashRef Any Int Str);
 
     my %USERS = (
         1 => {
-            name => 'Darth Wader',
+            first_name => 'Darth',
+            last_name => 'Wader',
             password => 'deathstar',
             email => 'darth@deathstar.com',
         },
         2 => {
-            name => 'Luke Skywalker',
+            first_name => 'Luke',
+            last_name => 'Skywalker',
             password => 'qwerty',
             email => 'l.skywalker@jedi.com',
         },
     );
 
     plugin 'Swagger', enable => 'CORS';
-    api_format 'json';
+    #api_format 'json';
 
     swagger_setup(
         title => 'A POD synopsis API',
@@ -342,8 +347,8 @@ Raisin - a REST API micro framework for Perl.
     resource users => sub {
         summary 'List users';
         params(
-            optional => { name => 'start', type => Int, default => 0, desc => 'Pager (start)' },
-            optional => { name => 'count', type => Int, default => 10, desc => 'Pager (count)' },
+            optional('start', type => Int, default => 0, desc => 'Pager (start)'),
+            optional('count', type => Int, default => 10, desc => 'Pager (count)'),
         );
         get sub {
             my $params = shift;
@@ -370,21 +375,24 @@ Raisin - a REST API micro framework for Perl.
 
         summary 'Create new user';
         params(
-            requires => { name => 'name', type => Str, desc => 'User name' },
-            requires => { name => 'password', type => Str, desc => 'User password' },
-            optional => { name => 'email', type => Str, default => undef, regex => qr/.+\@.+/, desc => 'User email' },
+            requires('user', type => HashRef, desc => 'User object', group {
+                requires('first_name', type => Str, desc => 'First name'),
+                requires('last_name', type => Str, desc => 'Last name'),
+                requires('password', type => Str, desc => 'User password'),
+                optional('email', type => Str, default => undef, regex => qr/.+\@.+/, desc => 'User email'),
+            }),
         );
         post sub {
             my $params = shift;
 
             my $id = max(keys %USERS) + 1;
-            $USERS{$id} = $params;
+            $USERS{$id} = $params->{user};
 
             { success => 1 }
         };
 
         desc 'Actions on the user';
-        params requires => { name => 'id', type => Int, desc => 'User ID' };
+        params requires('id', type => Int, desc => 'User ID');
         route_param 'id' => sub {
             summary 'Show user';
             get sub {
@@ -398,15 +406,6 @@ Raisin - a REST API micro framework for Perl.
                 { success => delete $USERS{ $params->{id} } };
             };
         };
-
-        summary 'NOP';
-        get nop => sub { };
-    };
-
-    desc 'Echo API endpoint';
-    resource echo => sub {
-        params(optional => { name => 'data0', type => Any, default => "ёй" });
-        get sub { shift };
     };
 
     run;
@@ -446,15 +445,15 @@ Shortcuts to add a C<route> restricted to the corresponding HTTP method.
     del 'all' => sub { 'OK' };
 
     params(
-        requires => { name => 'id', type => Int },
-        optional => { name => 'key', type => Str },
+        requires('id', type => Int),
+        optional('key', type => Str),
     );
     get sub { 'GET' };
 
     desc 'Put data';
     params(
-        required => { name => 'id', type => Int },
-        optional => { name => 'name', type => Str },
+        required('id', type => Int),
+        optional('name', type => Str),
     );
     put 'all' => sub {
         'PUT'
@@ -497,14 +496,14 @@ Here you can define validations and coercion options for your parameters.
 Can be applied to any HTTP method and/or C<route_param> to describe parameters.
 
     params(
-        requires => { name => 'name', type => Str },
-        optional => { name => 'start', type => Int, default => 0 },
-        optional => { name => 'count', type => Int, default => 10 },
+        requires('name', type => Str),
+        optional('start', type => Int, default => 0),
+        optional('count', type => Int, default => 10),
     );
     get sub { ... };
 
     params(
-        requires => { name => 'id', type => Int, desc => 'User ID' },
+        requires('id', type => Int, desc => 'User ID'),
     );
     route_param 'id' => sub { ... };
 
@@ -682,7 +681,7 @@ Query string and body parameters will be merged (see L<Plack::Request/parameters
 
 =head2 Declared parameters
 
-Raisin allows you to access only the parameters that have been declared by your
+Raisin allows you to access only the parameters that have been declared by you in
 L<Raisin/params> block.
 
 By default you can get all declared parameter as a first argument passed to your
@@ -705,16 +704,16 @@ Response:
 
     { "data": nil }
 
-Once we add parameters requirements, Raisin will start returning only
-the declared params.
+Once we add parameters block, Raisin will start return only the declared parameters.
 
 Application:
 
     api_format 'json';
 
-    params
-        required => { name => 'id', type => Int };
-        optional => { name => 'email', type => Str };
+    params(
+        requires('id', type => Int),
+        optional('email', type => Str)
+    );
     post data => sub {
         my $params = shift;
         { data => $params };
@@ -728,16 +727,17 @@ Response:
 
     { "data": { "id": 42 } }
 
-By default declared parameters doesn't contain parameters that has C<undef> value.
+By default declared parameters don't contain parameters which have no value.
 If you want to return all parameters you can use the C<include_missing> function.
 
 Application:
 
     api_format 'json';
 
-    params
-        required => { name => 'id', type => Int },
-        optional => { name => 'email', type => Str };
+    params(
+        requires('id', type => Int),
+        optional('email', type => Str)
+    );
     post data => sub {
         my $params = shift;
         { data => include_missing($params) };
@@ -756,12 +756,12 @@ Response:
 You can define validations and coercion options for your parameters using a
 L<Raisin/params> block.
 
-Parameters can C<requires> a value and can be an C<optional>.
-C<optional> parameters can have a default value.
+Parameters can C<requires> value or can be C<optional>.
+C<optional> parameters can have default value.
 
     params(
-        requires => { name => 'name', type => Str },
-        optional => { name => 'count', type => Int, default => 10 },
+        requires('name', type => Str),
+        optional('count', type => Int, default => 10),
     );
     get sub {
         my $params = shift;
@@ -789,6 +789,27 @@ Available arguments:
 
 =back
 
+=head2 Nested Parameters
+
+=head3 Hash
+
+Use a keyword C<group> to define a group of parameters which is enclosed to
+the parent C<HashRef> parameter.
+
+    params(
+        requires('name', type => HashRef, group {
+            requires('first_name', type => Str),
+            requires('last_name', type => Str),
+        })
+    )
+
+=head3 Array
+
+Use C<ArrayRef[*]> types from your compatible type library to define arrays.
+
+    requires('list', type => ArrayRef[Int], desc => 'List of integers')
+
+
 =head2 Types
 
 Raisin supports Moo(se)-compatible type constraint so you can use any of the
@@ -930,9 +951,9 @@ Including parameters:
     GET     /echo
       *data Any{ёй}
 
-=head2 Swagger
+=head2 OpenAPI/Swagger
 
-L<Swagger|https://github.com/wordnik/swagger-core> compatible API documentations.
+L<Swagger|http://swagger.io> compatible API documentations.
 
     plugin 'Swagger';
 
